%% Change Points in the Mean, Standard Deviation and Trend of CB01
% by M.H. Trauth 6 Nov 2017

%%
% Clear workspace, command window and close all figures. Load data from
% file. Remove rows with NaNs.
clear, clc, close all
CB_PC1_ages = load('CB_PC1_ages.txt');
CB_PC1_ages(isnan(CB_PC1_ages(:,1))==1,:) = [];

%% 
% Calculate avarage time spacing between subsequent data point. Define
% evenly spaced time vector and interpolate data.
min(CB_PC1_ages(:,1))
max(CB_PC1_ages(:,1))
min(diff(CB_PC1_ages(:,1)))
max(diff(CB_PC1_ages(:,1)))
mean(diff(CB_PC1_ages(:,1)))
std(diff(CB_PC1_ages(:,1)))

t = 0 : 15 : 45350;
y = interp1(CB_PC1_ages(:,1),CB_PC1_ages(:,2),t,'linear','extrap');

%% 
% Define number of change points for |findchangepts| from the Signal
% Processing Toolbox:
maxnumchg = 15;

%% 
% Parula colors.
colors = parula(8);

%% Changes in the measures of central tendency
% Now we use the function |findchangepts| from the Signal Processing
% Toolbox instead. We first determine change points in the mean.
findchangepts(y,'MaxNumChanges',maxnumchg)
[ipoint,residual] = findchangepts(y,'MaxNumChanges',maxnumchg);
xlabel('Index')
ylabel('K (standardized cps)')
title('Result from findchangepoint test for the mean');
close

m(1) = mean(y(1:ipoint(1)));
for i = 1 : length(ipoint)-1
    m(i+1) = mean(y(ipoint(i):ipoint(i+1)));
end
m(length(ipoint)+1) = mean(y(ipoint(i+1):end));

fig = figure('Color',[1 1 1],...
    'Position',[50 1000 1200 300]);
axes('Box','On',...
    'YDir','Reverse',...
    'YLim',[-6 6],...
    'XGrid','On',...
    'FontSize',14,...
    'YGrid','On',...
    'LineWidth',1); hold on
line(t/1000,y,'LineWidth',1);
line([0 t(ipoint(1))]/1000,[m(1) m(1)],...
    'Color',[0.8510 0.3255 0.0980],...
    'LineWidth',1)
for i = 1 : length(ipoint)-1
    line([t(ipoint(i)) t(ipoint(i+1))]/1000,[m(i+1) m(i+1)],...
    'Color',[0.8510 0.3255 0.0980],...
    'LineWidth',1)
end
line([t(ipoint(length(ipoint)))/1000 t(end)/1000],[m(end) m(end)],...
    'Color',[0.8510 0.3255 0.0980],...
    'LineWidth',1)
line(t/1000,y,'LineWidth',1);
stem(t(ipoint)/1000,6*ones(size(t(ipoint))),...
    'BaseValue',-6,...
    'Color',[0.4667 0.6745 0.1882],...
    'Marker','none',...
    'LineWidth',1)
xlabel('Age (ka)')
ylabel('K (PC1 Scores)')
text(48.000,-5,'A',...
    'FontSize',24)

orient(fig,'landscape')
print(fig,'trauth_figure_5a.pdf','-dpdf','-bestfit')

%% Changes in the measures of dispersion
% Now we use the function |findchangepts| from the Signal Processing
% Toolbox instead. We first determine change points in the standard
% deviation.
findchangepts(y,'MaxNumChanges',maxnumchg,'Statistic','std')
[ipoint,residual] = findchangepts(y,...
    'MaxNumChanges',maxnumchg,'Statistic','std');
xlabel('Index')
ylabel('K (standardized cps)')
title('Result from findchangepoint test for the standard deviation');
close

s(1) = std(y(1:ipoint(1)));
for i = 1 : length(ipoint)-1
    s(i+1) = std(y(ipoint(i):ipoint(i+1)));
end
s(length(ipoint)+1) = std(y(ipoint(i+1):end));

fig = figure('Color',[1 1 1],...
    'Position',[50 600 1200 300]);
axes('XGrid','On',...
    'YGrid','On',...
    'YColor',[0.8510 0.3255 0.0980],...
    'YAxisLocation','right',...
    'Color','None',...
    'FontSize',14,...
    'YLim',[-0 3],...
    'LineWidth',1); hold on
xlabel('Age (ka)')
ylabel('Standard Deviation')
line([0 t(ipoint(1))]/1000,[s(1) s(1)],...
    'Color',[0.8510 0.3255 0.0980],...
    'LineWidth',1)
for i = 1 : length(ipoint)-1
    line([t(ipoint(i)) t(ipoint(i+1))]/1000,[s(i+1) s(i+1)],...
    'Color',[0.8510 0.3255 0.0980],...
    'LineWidth',1)
end
line([t(ipoint(length(ipoint))) t(end)]/1000,[s(end) s(end)],...
    'Color',[0.8510 0.3255 0.0980],...
    'LineWidth',1)
xlabel('Age (ka)')
ylabel('Standard Deviation')
axes('Color','None',...
    'Box','On',...
    'YDir','Reverse',...
    'YLim',[-6 6],...
    'FontSize',14,...
    'XGrid','On',...
    'YGrid','On',...
    'LineWidth',1); hold on
line(t/1000,y,...
    'LineWidth',1);
stem(t(ipoint)/1000,6*ones(size(t(ipoint))),...
    'BaseValue',-6,...
    'Color',[0.4667 0.6745 0.1882],...
    'Marker','none',...
    'LineWidth',1)
xlabel('Age (ka)')
ylabel('K (PC1 Scores)')
text(48.000,-5,'B',...
    'FontSize',24)

orient(fig,'landscape')
print(fig,'trauth_figure_5b.pdf','-dpdf','-bestfit')

%% Changes in the trend
% Now we use the function |findchangepts| from the Signal Processing
% Toolbox instead. We first determine change points in the trend.
findchangepts(y,'MaxNumChanges',maxnumchg,'Statistic','linear')
[ipoint,residual] = findchangepts(y,...
    'MaxNumChanges',maxnumchg,'Statistic','linear');
xlabel('Index')
ylabel('K (standardized cps)')
title('Result from findchangepoint test for the trend');
close

clear p
p(1,:) = polyfit(t(1:ipoint(1)),y(1:ipoint(1)),1);
for i = 1 : length(ipoint)-1
    p(i+1,:) = polyfit(t(ipoint(i):ipoint(i+1)),...
        y(ipoint(i):ipoint(i+1)),1);
end
p(length(ipoint)+1,:) = polyfit(t(ipoint(length(ipoint)):end),...
    y(ipoint(length(ipoint)):end),1);

fig = figure('Color',[1 1 1],...
    'Position',[50 200 1200 300]);
axes('Box','On',...
    'YDir','Reverse',...
    'YLim',[-6 6],...
    'XGrid','On',...
    'FontSize',14,...
    'YGrid','On',...
     'LineWidth',1); hold on
line(t/1000,y,'LineWidth',1);
line(t(1:ipoint(1))/1000,polyval(p(1,:),t(1:ipoint(1))),...
    'Color',[0.8510 0.3255 0.0980],...
    'LineWidth',1)
for i = 1 : length(ipoint)-1
    line(t(ipoint(i:i+1))/1000,polyval(p(i+1,:),t(ipoint(i:i+1))),...
    'Color',[0.8510 0.3255 0.0980],...
    'LineWidth',1)
end
line(t(ipoint(length(ipoint)):end)/1000,polyval(p(end,:),...
    t(ipoint(length(ipoint)):end)),...
    'Color',[0.8510 0.3255 0.0980],...
    'LineWidth',1)
stem(t(ipoint)/1000,6*ones(size(t(ipoint))),...
    'BaseValue',-6,...
    'Color',[0.4667 0.6745 0.1882],...
    'Marker','none',...
    'LineWidth',1)
xlabel('Age (ka)')
ylabel('K (PC1 Scores)')
text(48.000,-5,'C',...
    'FontSize',24)

orient(fig,'landscape')
print(fig,'trauth_figure_5c.pdf','-dpdf','-bestfit')


