%% Time Warping of Chew Bahir Cores CB01-CB06
% by M.H. Trauth 6 Nov 2017

%%
% Clear workspace, clear Command Window, close all figures and load the
% data.
clear, clc, close all
load CBALL_xrf.mat

%% 
% All potassium records are standardized to |mean=0| and |std=1|.
CB01(:,1) = CB01xrf(:,1);
CB02(:,1) = CB02xrf(:,1)/10;
CB03(:,1) = CB03xrf(:,1)/10;
CB04(:,1) = CB04xrf(:,1)/10;
CB05(:,1) = CB05xrf(:,1)/10;
CB06(:,1) = CB06xrf(:,1)/10;
CB01(:,2) = (CB01xrf(:,8)-nanmean(CB01xrf(:,8)))./nanstd(CB01xrf(:,8));
CB02(:,2) = (CB02xrf(:,18)-nanmean(CB02xrf(:,18)))./nanstd(CB02xrf(:,18));
CB03(:,2) = (CB03xrf(:,15)-nanmean(CB03xrf(:,15)))./nanstd(CB03xrf(:,15));
CB04(:,2) = (CB04xrf(:,16)-nanmean(CB04xrf(:,16)))./nanstd(CB04xrf(:,16));
CB05(:,2) = (CB05xrf(:,15)-nanmean(CB05xrf(:,15)))./nanstd(CB05xrf(:,15));
CB06(:,2) = (CB06xrf(:,16)-nanmean(CB06xrf(:,16)))./nanstd(CB06xrf(:,16));

%% 
% Display the records vs. depth.
colors = parula(8);
fig = figure('Color',[1 1 1],...
     'Position',[50 150 1200 500]);
axes('Position',[0.10 0.70 0.8 0.15],...
     'Color','None',...
     'XLim',[0 2000],...
     'XTick',[],...
     'XColor','None',...
     'FontSize',14,...
     'YDir','reverse',...
     'YColor',colors(1,:),...
     'YLim',[-3 3],...
     'LineWidth',1), hold on
line(CB01(:,1),CB01(:,2),...
     'LineWidth',1,...
     'Color',colors(1,:))
text(1420,-2,'CB01',...
     'Color',colors(1,:),...
     'FontSize',14)
ylabel('K (cps)',...
     'Color',colors(1,:),...
     'FontSize',14)
axes('Position',[0.10 0.58 0.8 0.15],...
     'Color','None',...
     'XLim',[0 2000],...
     'XTick',[],...
     'XColor','None',...
     'FontSize',14,...
     'YDir','reverse',...
     'YColor',colors(2,:),...
     'YAxisLocation','right',...
     'LineWidth',1), hold on
line(CB02(:,1),CB02(:,2),...
     'LineWidth',1,...
     'Color',colors(2,:))
text(940,2,'CB02',...
     'Color',colors(2,:),...
     'FontSize',14)
ylabel('K (cps)',...
     'Color',colors(2,:),...
     'FontSize',14)
axes('Position',[0.10 0.46 0.8 0.15],...
     'Color','None',...
     'XLim',[0 2000],...
     'XTick',[],...
     'FontSize',14,...
     'XColor','None',...
     'YDir','reverse',...
     'YColor',colors(3,:),...
     'LineWidth',1), hold on
line(CB03(:,1),CB03(:,2),...
     'LineWidth',1,...
     'Color',colors(3,:))
text(1130,1.5,'CB03',...
     'Color',colors(3,:),...
     'FontSize',14)
ylabel('K (cps)',...
     'Color',colors(3,:),...
     'FontSize',14)
axes('Position',[0.10 0.34 0.8 0.15],...
     'Color','None',...
     'XLim',[0 2000],...
     'XTick',[],...
     'XColor','None',...
     'FontSize',14,...
     'YDir','reverse',...
     'YColor',colors(4,:),...
     'YAxisLocation','right',...
     'LineWidth',1), hold on
line(CB06(:,1),CB06(:,2),...
     'LineWidth',1,...
     'Color',colors(4,:))
text(1050,1.5,'CB06',...
     'Color',colors(4,:),...
     'FontSize',14)
ylabel('K (cps)',...
     'Color',colors(4,:),...
     'FontSize',14)
axes('Position',[0.10 0.22 0.8 0.15],...
     'Color','None',...
     'XLim',[0 2000],...
     'XTick',[],...
     'XColor','None',...
     'FontSize',14,...
     'YDir','reverse',...
     'YColor',colors(5,:),...
     'LineWidth',1), hold on
line(CB04(:,1),CB04(:,2),...
     'LineWidth',1,...
     'Color',colors(5,:))
text(1050,1.5,'CB04',...
     'Color',colors(5,:),...
     'FontSize',14)
ylabel('K (cps)',...
     'Color',colors(5,:),...
     'FontSize',14)
axes('Position',[0.10 0.1 0.8 0.15],...
     'Color','None',...
     'XLim',[0 2000],...
     'XColor','k',...
     'YDir','reverse',...
     'FontSize',14,...
     'YColor',colors(6,:),...
     'YAxisLocation','right',...
     'LineWidth',1), hold on
line(CB05(:,1),CB05(:,2),...
     'LineWidth',1,...
     'Color',colors(6,:))
text(1050,0,'CB05',...
     'Color',colors(6,:),...
     'FontSize',14)
ylabel('K (cps)',...
     'Color',colors(6,:),...
     'FontSize',14)
xlabel('Core Depth (cm)',...
     'FontSize',14)
 
orient(fig,'landscape')
print(fig,'trauth_figure_2.pdf','-dpdf','-bestfit')

%% 
% Eliminate rows with no data values.
CB01(isnan(CB01(:,2))==1,:) = [];
CB02(isnan(CB02(:,2))==1,:) = [];
CB03(isnan(CB03(:,2))==1,:) = [];
CB04(isnan(CB04(:,2))==1,:) = [];
CB05(isnan(CB05(:,2))==1,:) = [];
CB06(isnan(CB06(:,2))==1,:) = [];

%% 
% Eliminate duplicate data points.
for i = 1:length(CB01)-1
    if CB01(i,1) == CB01(i+1,1), CB01(i,1) = NaN; end
end
CB01(isnan(CB01(:,1))==1,:) = [];
for i = 1:length(CB02)-1
    if CB02(i,1) == CB02(i+1,1), CB02(i,1) = NaN; end
end
CB02(isnan(CB02(:,1))==1,:) = [];
for i = 1:length(CB03)-1
    if CB03(i,1) == CB03(i+1,1), CB03(i,1) = NaN; end
end
CB03(isnan(CB03(:,1))==1,:) = [];
for i = 1:length(CB04)-1
    if CB04(i,1) == CB04(i+1,1), CB04(i,1) = NaN; end
end
CB04(isnan(CB04(:,1))==1,:) = [];
for i = 1:length(CB05)-1
    if CB05(i,1) == CB05(i+1,1), CB05(i,1) = NaN; end
end
CB05(isnan(CB05(:,1))==1,:) = [];
for i = 1:length(CB06)-1
    if CB06(i,1) == CB06(i+1,1), CB06(i,1) = NaN; end
end
CB06(isnan(CB06(:,1))==1,:) = [];

%% 
% Fill arrays up to the same length of CB01.
CB01(end+1:size(CB01,1),:) = 0;
CB02(end+1:size(CB01,1),:) = 0;
CB03(end+1:size(CB01,1),:) = 0;
CB04(end+1:size(CB01,1),:) = 0;
CB05(end+1:size(CB01,1),:) = 0;
CB06(end+1:size(CB01,1),:) = 0;

%% 
% Define interval used in time warping. Using imax = size(CB01,1); uses the
% full length of the longest record which is CB01.
imax = size(CB01,1);

%% 
% Define graphics properties.
clear CB
colors = parula(8);
offset = 2;

%% Dynamic Time Warping using |dtw| from the Signal Processing Toolbox
% According to help, |dist = dtw(x,y)| stretches two vectors, |x| and |y|,
% onto a common set of instants such that dist, the sum of the Euclidean
% distances between corresponding points, is smallest. To stretch the
% inputs, |dtw| repeats each element of |x| and |y| as many times as
% necessary. If |x| and |y| are matrices, then |dtw| stretches them by
% repeating their columns. In that case, |x| and |y| must have the same
% number of rows. |[dist,ix,iy] = dtw(x,y)| returns the common set of
% instants, or warping path, such that |x(ix)| and |y(iy)| have the
% smallest possible |dist| between them. The vectors |ix| and |iy| have the
% same length. Each contains a monotonically increasing sequence in which
% the indices to the elements of the corresponding signal, |x| or |y|, are
% repeated the necessary number of times. When |x| and |y| are matrices,
% |ix| and |iy| are such that |x(:,ix)| and |y(:,iy)| are minimally
% separated. As the code for the previous experiment also |dtw| uses the
% concept of Dynamic Time Warping to Sakoe and Chiba (1978). In |dtw| it is
% important to adjust the |maxsamp|, which is the width of the adjustment
% window, specified as a positive integer. The distance metric is
% |euclidean| (Euclidean) by default, which can be changed to |absolute|
% (sum of absolut differences, also known as Manhattan or cityblock
% distance), |squared| (squared Euclidean) or |symmkl| (symmetric
% Kullback-Leibler metric).
maxsamp = 500;
metric = 'euclidean';

[~,ixCB01_1,iyCB02_1]=dtw(CB01(1:imax,2),CB02(1:imax,2),maxsamp,metric);
fig = figure('Color',[1 1 1],...
     'Position',[50 150 1200 800]);
axes('Box','On',...
     'YDir','Reverse',...
     'FontSize',14,...
     'LineWidth',1,...
     'XGrid','On',...
     'YGrid','On',...
     'XTick',0:200:3000), hold on
line(ixCB01_1,CB01(ixCB01_1,2),...
     'Color',colors(1,:),...
     'LineWidth',1)
line(ixCB01_1,CB02(iyCB02_1,2)+1*offset,...
     'Color',colors(2,:),...
     'LineWidth',1)
[~,ixCB01_2,iyCB03_2]=dtw(CB01(1:imax,2),CB03(1:imax,2),maxsamp,metric);
line(ixCB01_2,CB03(iyCB03_2,2)+2*offset,...
     'Color',colors(3,:),...
     'LineWidth',1)
[~,ixCB01_3,iyCB06_3]=dtw(CB01(1:imax,2),CB06(1:imax,2),maxsamp,metric);
line(ixCB01_3,CB06(iyCB06_3,2)+3*offset,...
     'Color',colors(4,:),...
     'LineWidth',1)
[~,ixCB01_4,iyCB04_4]=dtw(CB01(1:imax,2),CB04(1:imax,2),maxsamp,metric);
line(ixCB01_4,CB04(iyCB04_4,2)+4*offset,...
     'Color',colors(5,:),...
     'LineWidth',1)
[~,ixCB01_5,iyCB05_5]=dtw(CB01(1:imax,2),CB05(1:imax,2),maxsamp,metric);
line(ixCB01_5,CB05(iyCB05_5,2)+5*offset,...
     'Color',colors(6,:),...
     'LineWidth',1)
text(2800,0,'CB01',...
     'Color',colors(1,:),...
     'FontSize',14)
text(2800,2,'CB02',...
     'Color',colors(2,:),...
     'FontSize',14)
text(2800,4,'CB03',...
     'Color',colors(3,:),...
     'FontSize',14)
text(2800,6,'CB06',...
     'Color',colors(4,:),...
     'FontSize',14)
text(2800,8,'CB04',...
     'Color',colors(5,:),...
     'FontSize',14)
text(2800,10,'CB05',...
     'Color',colors(6,:),...
     'FontSize',14)
xlabel('Index')

%% 
fig = figure('Color',[1 1 1],...
     'Position',[50 150 1200 800]);
axes('Box','On',...
     'YDir','Reverse',...
     'FontSize',14,...
     'LineWidth',1,...
     'XGrid','On',...
     'YGrid','On',...
     'XTick',0:200:3000,...
     'YTick',0:200:3000), hold on
line(ixCB01_1,iyCB02_1,...
     'Color',colors(2,:),...
     'LineWidth',1)
line(ixCB01_2,iyCB03_2,...
     'Color',colors(3,:),...
     'LineWidth',1)
line(ixCB01_3,iyCB06_3,...
     'Color',colors(4,:),...
     'LineWidth',1)
line(ixCB01_4,iyCB04_4,...
     'Color',colors(5,:),...
     'LineWidth',1)
line(ixCB01_5,iyCB05_5,...
     'Color',colors(6,:),...
     'LineWidth',1)
line([0 imax],[0 imax],...
     'Color',[0.8 0.5 0.3],...
     'LineWidth',1,...
     'LineStyle',':')
xlabel('Index CB01')
ylabel('Index CB02-06')
legend({'CB01/02','CB01/03','CB01/06','CB01/04','CB01/05'},'Box','Off')

%% 
% Next we create new arrays CB01_cd to CB06_cd. Then we remove duplicate
% values. Then interpolate the data upon an evenly spaced index axis.
CB01_cd(:,1) = ixCB01_1;   CB01_cd(:,2) = CB01(ixCB01_1,2);
CB02_cd(:,1) = ixCB01_1;   CB02_cd(:,2) = CB02(iyCB02_1,2);
CB03_cd(:,1) = ixCB01_2;   CB03_cd(:,2) = CB03(iyCB03_2,2);
CB06_cd(:,1) = ixCB01_3;   CB06_cd(:,2) = CB06(iyCB06_3,2);
CB04_cd(:,1) = ixCB01_4;   CB04_cd(:,2) = CB04(iyCB04_4,2);
CB05_cd(:,1) = ixCB01_5;   CB05_cd(:,2) = CB05(iyCB05_5,2);

for i = 1:length(CB01_cd)-1
    if CB01_cd(i,1) == CB01_cd(i+1,1)
       CB01_cd(i,:) = NaN;
    end
end
CB01_cd(isnan(CB01_cd(:,1))==1,:) = [];

for i = 1:length(CB02_cd)-1
    if CB02_cd(i,1) == CB02_cd(i+1,1)
       CB02_cd(i,:) = NaN;
    end
end
CB02_cd(isnan(CB02_cd(:,1))==1,:) = [];

for i = 1:length(CB03_cd)-1
    if CB03_cd(i,1) == CB03_cd(i+1,1)
       CB03_cd(i,:) = NaN;
    end
end
CB03_cd(isnan(CB03_cd(:,1))==1,:) = [];

for i = 1:length(CB06_cd)-1
    if CB06_cd(i,1) == CB06_cd(i+1,1)
       CB06_cd(i,:) = NaN;
    end
end
CB06_cd(isnan(CB06_cd(:,1))==1,:) = [];

for i = 1:length(CB04_cd)-1
    if CB04_cd(i,1) == CB04_cd(i+1,1)
       CB04_cd(i,:) = NaN;
    end
end
CB04_cd(isnan(CB04_cd(:,1))==1,:) = [];

for i = 1:length(CB05_cd)-1
    if CB05_cd(i,1) == CB05_cd(i+1,1)
       CB05_cd(i,:) = NaN;
    end
end
CB05_cd(isnan(CB05_cd(:,1))==1,:) = [];

CB01_int(:,1) = 0 : 0.5 : imax;
CB01_int(:,2) = interp1(CB01_cd(:,1),...
    CB01_cd(:,2),CB01_int(:,1),'linear','extrap');
CB02_int(:,1) = 0 : 0.5 : imax;
CB02_int(:,2) = interp1(CB02_cd(:,1),...
    CB02_cd(:,2),CB02_int(:,1),'linear','extrap');
CB03_int(:,1) = 0 : 0.5 : imax;
CB03_int(:,2) = interp1(CB03_cd(:,1),...
    CB03_cd(:,2),CB03_int(:,1),'linear','extrap');
CB06_int(:,1) = 0 : 0.5 : imax;
CB06_int(:,2) = interp1(CB06_cd(:,1),...
    CB06_cd(:,2),CB06_int(:,1),'linear','extrap');
CB04_int(:,1) = 0 : 0.5 : imax;
CB04_int(:,2) = interp1(CB04_cd(:,1),...
    CB04_cd(:,2),CB04_int(:,1),'linear','extrap');
CB05_int(:,1) = 0 : 0.5 : imax;
CB05_int(:,2) = interp1(CB05_cd(:,1),...
    CB05_cd(:,2),CB05_int(:,1),'linear','extrap');

%%
CB01_int(:,1) = CB01(imax,1)*CB01_int(:,1)/imax;
CB02_int(:,1) = CB01(imax,1)*CB02_int(:,1)/imax;
CB03_int(:,1) = CB01(imax,1)*CB03_int(:,1)/imax;
CB06_int(:,1) = CB01(imax,1)*CB06_int(:,1)/imax;
CB04_int(:,1) = CB01(imax,1)*CB04_int(:,1)/imax;
CB05_int(:,1) = CB01(imax,1)*CB05_int(:,1)/imax;

%% 
% Calibrating time warping results to radiocarbon age model.
load agemodel.mat
CB01_ages(:,1) = interp1(agemodel(:,1),agemodel(:,2),...
    CB01_int(:,1),'linear','extrap');
CB01_ages(:,2) = CB01_int(:,2);
CB02_ages(:,1) = interp1(agemodel(:,1),agemodel(:,2),...
    CB02_int(:,1),'linear','extrap');
CB02_ages(:,2) = CB02_int(:,2);
CB03_ages(:,1) = interp1(agemodel(:,1),agemodel(:,2),...
    CB03_int(:,1),'linear','extrap');
CB03_ages(:,2) = CB03_int(:,2);
CB06_ages(:,1) = interp1(agemodel(:,1),agemodel(:,2),...
    CB06_int(:,1),'linear','extrap');
CB06_ages(:,2) = CB06_int(:,2);
CB04_ages(:,1) = interp1(agemodel(:,1),agemodel(:,2),...
    CB04_int(:,1),'linear','extrap');
CB04_ages(:,2) = CB04_int(:,2);
CB05_ages(:,1) = interp1(agemodel(:,1),agemodel(:,2),...
    CB05_int(:,1),'linear','extrap');
CB05_ages(:,2) = CB05_int(:,2);

%%
% Create Figure 3
fig = figure('Color',[1 1 1],...
     'Position',[50 150 1200 800]);
axes('Box','On',...
     'YDir','Reverse',...
     'FontSize',14,...
     'LineWidth',1,...
     'XGrid','On',...
     'YGrid','On',...
     'YLim',[-5 12]), hold on
line(CB01_ages(:,1)/1000,CB01_ages(:,2),...
     'Color',colors(1,:),...
     'LineWidth',1)
line(CB02_ages(:,1)/1000,CB02_ages(:,2)+1*offset,...
     'Color',colors(2,:),...
     'LineWidth',1)
line(CB03_ages(:,1)/1000,CB03_ages(:,2)+2*offset,...
     'Color',colors(3,:),...
     'LineWidth',1)
line(CB06_ages(:,1)/1000,CB06_ages(:,2)+3*offset,...
     'Color',colors(4,:),...
     'LineWidth',1)
line(CB04_ages(:,1)/1000,CB04_ages(:,2)+4*offset,...
     'Color',colors(5,:),...
     'LineWidth',1)
line(CB05_ages(:,1)/1000,CB05_ages(:,2)+5*offset,...
     'Color',colors(6,:),...
     'LineWidth',1)
 
text(45.500,0,'CB01',...
     'Color',colors(1,:),...
     'FontSize',14)
text(45.500,2,'CB02',...
     'Color',colors(2,:),...
     'FontSize',14)
text(45.500,4,'CB03',...
     'Color',colors(3,:),...
     'FontSize',14)
text(45.500,6,'CB06',...
     'Color',colors(4,:),...
     'FontSize',14)
text(45.500,8,'CB04',...
     'Color',colors(5,:),...
     'FontSize',14)
text(45.500,10,'CB05',...
     'Color',colors(6,:),...
     'FontSize',14)

line([4.875 15.460],[-3.5 -3.5],...
     'LineWidth',1,...
     'Color',[0 0 0])
text(10.168,-4,...
     'African Humid Period',...
     'HorizontalAlignment','Center',...
     'FontSize',14)
line([11.730 13.270],[-2.2 -2.2],...
     'LineWidth',1,...
     'Color',[0 0 0])
text(12.500,-2.7,...
     'YD',...
     'HorizontalAlignment','Center',...
     'FontSize',14)

xlabel('Age (ka)')
ylabel('K (normalized cps)',...
     'FontSize',14)

orient(fig,'landscape')
print(fig,'trauth_figure_3.pdf','-dpdf','-bestfit')

%% Principal component analysis
% We run a PCA to extract the regional signal from the six cores
age = CB01_ages(:,1);
CBxrfdtw(:,1) =  CB01_ages(:,2);
CBxrfdtw(:,2) =  CB02_ages(:,2);
CBxrfdtw(:,3) =  CB03_ages(:,2);
CBxrfdtw(:,4) =  CB06_ages(:,2);
CBxrfdtw(:,5) =  CB04_ages(:,2);
CBxrfdtw(:,6) =  CB05_ages(:,2);

%%
% Run the PCA
[pcs,newx,variances] = pca(CBxrfdtw);
percent_explained = 100*variances/sum(variances)

%%
% PCA results
fig = figure('Color',[1 1 1],...
     'Position',[50 150 1200 800]);
axes('Box','On',...
     'YDir','Reverse',...
     'FontSize',14,...
     'LineWidth',1,...
     'YLim',[-8 12],...
     'XGrid','On',...
     'YGrid','On'), hold on
line(age/1000,newx(:,1),'Color',colors(1,:),...
     'LineWidth',1)
line(age/1000,newx(:,2)+1*offset,...
     'Color',colors(2,:),...
     'LineWidth',1)
line(age/1000,newx(:,3)+2*offset,...
     'Color',colors(3,:),...
     'LineWidth',1)
line(age/1000,newx(:,4)+3*offset,...
     'Color',colors(4,:),...
     'LineWidth',1)
line(age/1000,newx(:,5)+4*offset,...
     'Color',colors(5,:),...
     'LineWidth',1)
line(age/1000,newx(:,6)+5*offset,...
     'Color',colors(6,:),...
     'LineWidth',1)
text(46.000,0,'PC1',...
     'Color',colors(1,:),...
     'FontSize',14)
text(46.000,2,'PC2',...
     'Color',colors(2,:),...
     'FontSize',14)
text(46.000,4,'PC3',...
     'Color',colors(3,:),...
     'FontSize',14)
text(46.000,6,'PC4',...
     'Color',colors(4,:),...
     'FontSize',14)
text(46.000,8,'PC5',...
     'Color',colors(5,:),...
     'FontSize',14)
text(46.000,10,'PC6',...
     'Color',colors(6,:),...
     'FontSize',14)
xlabel('Age (ka)')
ylabel('K (PCA scores)',...
     'FontSize',14)

line([4.875 15.460],[-6 -6],...
     'LineWidth',1,...
     'Color',[0 0 0])
text(10.168,-6.5,...
     'African Humid Period',...
     'HorizontalAlignment','Center',...
     'FontSize',14)
line([11.730 13.270],[-4.3 -4.3],...
     'LineWidth',1,...
     'Color',[0 0 0])
text(12.500,-4.8,...
     'YD',...
     'HorizontalAlignment','Center',...
     'FontSize',14) 
 
orient(fig,'landscape')
print(fig,'trauth_figure_4.pdf','-dpdf','-bestfit')

%%
% PC1 of PCA result only - very nice!!!
% fig = figure('Color',[1 1 1],...
%      'Position',[50 150 1200 300]);
% axes('Box','On',...
%      'YDir','Reverse',...
%      'FontSize',14,...
%      'LineWidth',1,...
%      'XGrid','On',...
%      'YGrid','On'), hold on
% line(age,newx(:,1),...
%      'Color',colors(1,:),...
%      'LineWidth',1)
% text(46.000,0,'PC1',...
%      'Color',colors(1,:),...
%      'FontSize',14)
% xlabel('Age (ka)')

%%
% Export PC1 results
data(:,1) = age;
data(:,2) = newx(:,1);
save CB_PC1_ages.txt data -ascii









